/*
** Copyright 2011, Paul Kocialkowski
** Based on: git://gitorious.org/~jbruneaux/xdandroid/hardware_msm7k_libacoustic.git (branch froyo)
** Credit: Jbruneaux
**
** Licensed under the Apache License, Version 2.0 (the "License");
** you may not use this file except in compliance with the License.
** You may obtain a copy of the License at
**
**     http://www.apache.org/licenses/LICENSE-2.0
**
** Unless required by applicable law or agreed to in writing, software
** distributed under the License is distributed on an "AS IS" BASIS,
** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
** See the License for the specific language governing permissions and
** limitations under the License.
*/

#define LOG_TAG "AudioAcousticMSM72XX"
#include <utils/Log.h>
#include <utils/String8.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <dlfcn.h>
#include <fcntl.h>
#include <malloc.h>

#include <time.h>
#include <pthread.h>
#include <string.h>
#include <termios.h>
#include <assert.h>

#include <ctype.h>

#include <errno.h>

#include <linux/msm_audio.h>

#include "AudioHardware.h"
#include "AudioAcoustic.h"

#define AUDIO_PARA_CUSTOM_FILENAME        "/sdcard/AudioPara4.csv"
#define AUDIO_PARA_DEFAULT_FILENAME       "/system/etc/AudioPara4.csv"
   
namespace android {

struct au_table_s {
    union {
        struct au_table_st table;
        char   array[0x80];
    };
};

struct be_table_s {
    union {
        struct be_table_st table;
        uint16_t array[0x80];
    };
};

struct d_table_s {
    union {
        struct d_table_st table;
        uint16_t array[0xB];        // Size = 0x8 on some devices like diamond.
    };
};

struct fg_table_s {
    union {
        struct fg_table_st table;
        uint16_t array[0xA0];
    };
};

struct c_table_s {
    union {
        struct fg_table_st table;
        uint16_t array[0x50];  
    };
};

/***********************************************************************************
 *
 *  Global variables
 *
 ***********************************************************************************/
static struct au_table_s*    Audio_Path_Table = NULL;              /* a_table ('A') */
static uint8_t APT_max_index   = 0;
static struct au_table_s*    Audio_Path_Uplink_Table = NULL;       /* u_table ('U') */
static uint8_t APUT_max_index  = 0;
static struct fg_table_s*    Phone_Acoustic_Table = NULL;          /* f_table ('F' + 'B') */
static uint8_t PAT_max_index   = 0;
static struct fg_table_s*    BT_Phone_Acoustic_Table = NULL;       /* g_table ('E' + 'G') */
static uint8_t BTPAT_max_index = 0;
static struct d_table_s*     HTC_VOC_CAL_CODEC_TABLE_Table = NULL; /* d_table ('D') */
static uint8_t HVCCT_max_index = 0;
static struct c_table_s*     CE_Acoustic_Table = NULL;             /* c_table ('C') */
static uint8_t CEAT_max_index  = 0;

/* Communication with kernel */
int acousticfd = 0;
int mNumSndEndpoints;
static struct msm_snd_endpoint *mSndEndpoints;
static struct msm_acoustic_capabilities device_capabilities;
int TPA2016fd;

static bool mInit = false;

static struct rx_iir_filter iir_cfg[1];
static struct adrc_filter adrc_cfg[1];
static struct eqalizer eqalizer[1];
static uint16_t adrc_flag[1];
static uint16_t eq_flag[1];
static uint16_t rx_iir_flag[1];
static bool audpp_filter_inited = false;
static bool audpre_filter_inited = false;
static bool adrc_filter_exists[1];

static struct tx_iir tx_iir_cfg[18];    // Normal + Full DUplex
static struct ns ns_cfg[9];
static struct tx_agc tx_agc_cfg[9];

// Current TPA2016 registers value initialized with default values
static uint8_t tpa2016d2_regs[7] = { 0xC3, 0x05, 0x0B, 0x00, 0x06, 0x3A, 0xC2 };
/* Values from TIAGC.csv */
static const uint8_t tpa2016d2_regs_audio[7] = { 0xC2, 0x20, 0x01, 0x00, 0x10, 0x19, 0xC0 };
static const uint8_t tpa2016d2_regs_voice[7] = { 0xC3, 0x3F, 0x01, 0x00, 0x14, 0x7A, 0xC0 };

static bool bCurrentAudioUplinkState = 0;
static bool bCurrentEnableHSSDState = 0;
static bool bCurrentAUXBypassReqState = 0;

int mCurrent_Adie_PGA_Gain = 1;

int openacousticfd(void)
{
    acousticfd = open("/dev/htc-acoustic", O_RDWR);
    if ( acousticfd < 0 ) {
        LOGE("Error opening dev %s (fd = %d). Error %s (%d)", "/dev/htc-acoustic", acousticfd,
                    strerror(errno), errno);
        return -1;
    } 
    return 0;
}


int get_device_capabilities(void)
{
    int bOn;
    if ( ioctl(acousticfd, ACOUSTIC_GET_CAPABILITIES, &device_capabilities) < 0) {
        LOGE("ACOUSTIC_GET_CAPABILITIES error.");
        return -EIO;
    } 
    LOGV("Device capabilities :");
    LOGV("- Htc voc cal fields per params : %d", device_capabilities.htc_voc_cal_fields_per_param);
    LOGV("- Dual mic supported : %s", (device_capabilities.bDualMicSupported)?"true":"false");

    /* Test for TPA2016 */
    if ( device_capabilities.bUseTPA2016 ) {
        TPA2016fd = open(MSM_TPA2016D2_DEV, O_RDWR);
        if ( TPA2016fd < 0 ) {
            LOGE("Error opening dev %s (fd = %d). Error %s (%d)", MSM_TPA2016D2_DEV, acousticfd,
                        strerror(errno), errno);
            LOGE("Device claims to have TPA2016 amplifier but %s not present. Disable TPA2016", MSM_TPA2016D2_DEV);
            device_capabilities.bUseTPA2016 = false;
        } else {
            /* Power on amplifier */
            bOn = 1;
            if (ioctl(TPA2016fd, TPA2016_SET_POWER, &bOn ) < 0) {
                LOGE("TPA2016_SET_POWER error.");
                return -EIO;
            }

            /* Read current device configuration */
            if (ioctl(TPA2016fd, TPA2016_READ_CONFIG, &tpa2016d2_regs ) < 0) {
                LOGE("TPA2016_READ_CONFIG error.");
                return -EIO;
            }  

            /* Power off amplifier */
            bOn = 0;
            if (ioctl(TPA2016fd, TPA2016_SET_POWER, &bOn ) < 0) {
                LOGE("TPA2016_SET_POWER error.");
                return -EIO;
            }
        } 
    }
    LOGV("- Use TPA2016 Amplifier : %s", (device_capabilities.bUseTPA2016)?"true":"false");
    return 0;
}

int get_pga_gain_for_fm_profile(int profile, int current_pga_gain)
{
    // FM HEADSET index
    if ( (profile == 18) | (profile == 19) ) {
        if ( current_pga_gain < 0xB ) {
            return ( 0x34 - (current_pga_gain * 4) );
        }
        return 0xFF; 
    }
    // FM SPEAKER index
    else if ( (profile == 22) | (profile == 23) ) {
        if ( current_pga_gain < 0xB ) {
            return ( 0x28 - (current_pga_gain * 4) );
        }
        return 0xFF; 
    }
    return 0;
}

int UpdateAudioAdieTable(bool bAudioUplinkReq, int paramR1, bool bEnableHSSD, bool bAUXBypassReq, bool bForceUpdate)
{
    struct adie_table table;
    int table_num = 0;
    int tab_byte_idx;

    char temp_table[0x80];
    uint16_t pcom_update[1];

    LOGV("UpdateAudioAdieTable(bAudioUplinkReq %d,bAUXBypassReq %d, bEnableHSSD=%d, bForceUpdate = %d)\n",
            bAudioUplinkReq, bEnableHSSD, bAUXBypassReq, bForceUpdate);

    if ( (bCurrentAudioUplinkState  == bAudioUplinkReq) && 
         (bCurrentEnableHSSDState   == bEnableHSSD ) &&
         (bCurrentAUXBypassReqState == bAUXBypassReq) && 
         (bForceUpdate == false) ) {
        LOGV("Update not required");
        return 0;
    }

    do 
    {
        memset(temp_table, 0, 0x80);
        tab_byte_idx = 0;
        
        do
        {
            temp_table[tab_byte_idx] = Audio_Path_Table[table_num].array[tab_byte_idx];
            temp_table[tab_byte_idx + 1] = (temp_table[tab_byte_idx + 1] | Audio_Path_Table[table_num].array[tab_byte_idx + 1]);

            if ( (!(table_num & 1)) && ( bAudioUplinkReq != 0 ) ) {
                temp_table[tab_byte_idx + 1] = (Audio_Path_Uplink_Table[table_num].array[tab_byte_idx + 1] 
                                                     | Audio_Path_Table[table_num].array[tab_byte_idx + 1]);
            }

            if ( paramR1 == 0 ) {
                if ( bEnableHSSD != 0 ) {
                    if ( Audio_Path_Table[table_num].array[tab_byte_idx] == 0x37 ) {
                        temp_table[tab_byte_idx + 1] |= 0x80;
                    }
                    if ( Audio_Path_Table[table_num].array[tab_byte_idx] == 0x48 ) {
                        temp_table[tab_byte_idx + 1] |= 0xC0;
                    }
                }
            } else {
                if ( Audio_Path_Table[table_num].array[tab_byte_idx] != 0x3E ) {
                    temp_table[tab_byte_idx + 1] = (temp_table[tab_byte_idx + 1] & 0xE7) | 0x10;           
                }
            }

            if ( (bAUXBypassReq != 0) && (Audio_Path_Table[table_num].array[tab_byte_idx] == 0x42) ) {
                temp_table[tab_byte_idx + 1] = get_pga_gain_for_fm_profile(table_num, mCurrent_Adie_PGA_Gain);
            }

            tab_byte_idx += 2;
        }
        while( tab_byte_idx < 0x80);
        
        /* Send table to kernel for update */
        table.table_num = table_num;
        table.pcArray = temp_table;
        if ( ioctl(acousticfd, ACOUSTIC_UPDATE_ADIE_TABLE, &table) < 0) {
            LOGE("ACOUSTIC_UPDATE_ADIE_TABLE error.");
            return -EIO;
        }    

        table_num += 1;
    }
    while ( table_num < APT_max_index );

    /* Generate PCOM_UPDATE_AUDIO 0x1 */
    struct audio_update_req req;
    req.type = PCOM_UPDATE_REQ;
    req.value = 0x1;

    if ( ioctl(acousticfd, ACOUSTIC_UPDATE_AUDIO_SETTINGS, &req) < 0) {
        LOGE("ACOUSTIC_UPDATE_AUDIO_SETTINGS error.");
        return -EIO;
    }     

    bCurrentAudioUplinkState  = bAudioUplinkReq;
    bCurrentEnableHSSDState   = bEnableHSSD;
    bCurrentAUXBypassReqState = bAUXBypassReq;

    return 0;
}


int ParseAudioParaLine(char* line, int len)
{
    char *token, *ps;
    int table_num;
    int field_count = 0;

    /* Parse the first field */
    token = strtok(line, ",");

    if(strcmp("Header", token) == 0)
    {
        token = strtok(NULL, ",");
        LOGI("CSV Header: %s\n", token);
        return 0;
    }

    switch ( token[0] ) {
        case 'A':
            table_num = strtol(token + 1, &ps, 10);
            if ( table_num > 31) {
                return -EINVAL;
            }
            if ( APT_max_index < (table_num+1) ) APT_max_index = table_num+1;
//            LOGI("Audio Path Table: %d\n", table_num);
            /* Skip the mode name string field */
            strtok(NULL, ",");
            while ( (token = strtok(NULL, ",")) ) {
                Audio_Path_Table[table_num].array[field_count++] = strtol(token, &ps, 16);
            };
            break;

        case 'B':
        case 'F':
            table_num = strtol(token + 1, &ps, 10);
            if ( table_num > 99) {
                return -EINVAL;
            }
            if ( PAT_max_index < (table_num+1) ) PAT_max_index = table_num+1;
//            LOGI("Phone Acoustic Table: %d\n", table_num);
            /* Skip the mode name string field */
            strtok(NULL, ",");
            while ( (token = strtok(NULL, ",")) ) {
                Phone_Acoustic_Table[table_num].array[field_count++] = strtol(token, &ps, 16);
            };
            break;

        case 'C':
            table_num = strtol(token + 1, &ps, 10);
            if ( table_num > 14) {
                return -EINVAL;
            }
            if ( CEAT_max_index < (table_num+1) ) CEAT_max_index = table_num+1;
//            LOGI("CE Acoustic Table: %d\n", table_num);
            /* Skip the mode name string field */
            strtok(NULL, ",");
            while ( (token = strtok(NULL, ",")) ) {
                CE_Acoustic_Table[table_num].array[field_count++] = strtol(token, &ps, 16);
            };
            break;

        case 'D':
            table_num = strtol(token + 1, &ps, 10);
            if ( table_num > 31) {
                return -EINVAL;
            }
            if ( HVCCT_max_index < (table_num+1) ) HVCCT_max_index = table_num+1;
//            LOGI("HTC_VOC_CAL_CODEC_TABLE Table: %d\n", table_num);
            /* Skip the mode name string field */
            strtok(NULL, ",");
            while ( (token = strtok(NULL, ",")) ) {
                HTC_VOC_CAL_CODEC_TABLE_Table[table_num].array[field_count++] = strtol(token, &ps, 16);
            };
            break;

        case 'E':
        case 'G':
#if 0
            table_num = strtol(token + 1, &ps, 10);
            if ( table_num > 100) {
                return -EINVAL;
            }
            if ( BTPAT_max_index < (table_num+1) ) BTPAT_max_index = table_num+1;
            //LOGV("BT Phone Acoustic Table: %d\n", table_num);
            /* Skip the mode name string field */
            strtok(NULL, ",");
            while ( (token = strtok(NULL, ",")) ) {
                BT_Phone_Acoustic_Table[table_num].array[field_count++] = strtol(token, &ps, 16);
            };
#endif
            /* Skip the table number field */
            token = strtok(NULL, ",");
            if ( BTPAT_max_index < 100 ) {  
                strncpy(BT_Phone_Acoustic_Table[BTPAT_max_index].table.name, token, MAX_MODE_NAME_LENGTH);          
//                LOGI("BT Phone Acoustic Table: %s\n", BT_Phone_Acoustic_Table[BTPAT_max_index].table.name);
                while ( (token = strtok(NULL, ",")) ) {
                    BT_Phone_Acoustic_Table[BTPAT_max_index].array[field_count++] = strtol(token, &ps, 16);
                };
                BTPAT_max_index++;
            }
            break;

        case 'H':
            //LOGV("It's just a header line, skip it\n");
            return 0;

        case 'U':
            table_num = strtol(token + 1, &ps, 10);
            if ( table_num > 31) {
                return -EINVAL;
            }
            if ( APUT_max_index < (table_num+1) ) APUT_max_index = table_num+1;
//            LOGI("Audio Path Table Uplink: %d\n", table_num);
            /* Skip the mode name string field */
            strtok(NULL, ",");
            while ( (token = strtok(NULL, ",")) ) {
                Audio_Path_Uplink_Table[table_num].array[field_count++] = strtol(token, &ps, 16);
            };
            break;

        default:
            LOGE("Unknown parameter field %c\n", token[0]);
            return -1;
    }
    
    return 0;

}

int ReadAudioParaFromFile(void)
{
    struct stat st;
    char *read_buf=NULL;
    char *next_str = NULL;
    char *current_str = NULL;
    int csvfd;
    struct htc_voc_cal_table htc_voc_cal_tbl;
    uint16_t* htc_voc_cal_tbl_conv = NULL;
        int len;

    static const char *path =
        AUDIO_PARA_CUSTOM_FILENAME;

    csvfd = open(path, O_RDONLY);
    if (csvfd < 0) {
        /* Failed to open custom parameters file, fallback to the default file ... */        
        LOGE("Failed to open %s. Error %s (%d)\n",
             path, strerror(errno), errno);

        LOGE("Trying with default file");
        path = AUDIO_PARA_DEFAULT_FILENAME;
        csvfd = open(path, O_RDONLY);
        if (csvfd < 0) {
            LOGE("Failed to open %s. Error %s (%d)\n",
                 path, strerror(errno), errno);
            return -1;
        }
    }
    
    LOGE("Successfully opened %s\n", path);

    if (fstat(csvfd, &st) < 0) {
        LOGE("Failed to stat %s: %s (%d)\n",
             path, strerror(errno), errno);
        close(csvfd);
        return -1;
    }


/*
    read_buf = (char *) mmap(0, st.st_size,
                    PROT_READ | PROT_WRITE,
                    MAP_PRIVATE,
                    csvfd, 0);
*/
    read_buf=(char *) malloc(st.st_size * sizeof(char));
    read(csvfd, read_buf, st.st_size * sizeof(char));
    current_str = read_buf;

    if (read_buf == MAP_FAILED) {
        LOGE("Failed to mmap parameters file: %s (%d)\n",
             strerror(errno), errno);
        close(csvfd);
        return -1;
    }

    if ( Audio_Path_Table == NULL ) {
        Audio_Path_Table = (struct au_table_s*) malloc(32 * sizeof(struct au_table_s) );  // 0x1000
        if (Audio_Path_Table == NULL) {
            LOGE("Failed to malloc Audio_Path_Table\n");
            return -1;
        }
    }

    if ( Audio_Path_Uplink_Table == NULL ) {
        Audio_Path_Uplink_Table = (struct au_table_s*) malloc(32 * sizeof(struct au_table_s) ); // 0x1000
        if (Audio_Path_Uplink_Table == NULL) {
            LOGE("Failed to malloc Audio_Path_Uplink_Table\n");
            return -1;
        }
    }

    if ( Phone_Acoustic_Table == NULL ) {
        Phone_Acoustic_Table = (struct fg_table_s*) malloc(100 * sizeof(struct fg_table_s) ); // 0x3C00
        if (Phone_Acoustic_Table == NULL) {
            LOGE("Failed to malloc Phone_Acoustic_Table\n");
            return -1;
        }
    }

    if ( BT_Phone_Acoustic_Table == NULL ) {
        BT_Phone_Acoustic_Table = (struct fg_table_s*) malloc(100 * sizeof(struct fg_table_s) );  // 0x7800
        if (BT_Phone_Acoustic_Table == NULL) {
            LOGE("Failed to malloc BT_Phone_Acoustic_Table\n");
            return -1;
        }
    }

    if ( HTC_VOC_CAL_CODEC_TABLE_Table == NULL ) {
        HTC_VOC_CAL_CODEC_TABLE_Table = (struct d_table_s*) malloc(32 * sizeof(struct d_table_s));
        if (HTC_VOC_CAL_CODEC_TABLE_Table == NULL) {
            LOGE("Failed to malloc HTC_VOC_CAL_CODEC_TABLE_Table\n");
            return -1;
        }
    }

    if ( CE_Acoustic_Table == NULL ) {
        CE_Acoustic_Table = (struct c_table_s*) malloc(15 * sizeof(struct c_table_s) );   // 0x960
        if (CE_Acoustic_Table == NULL) {
            LOGE("Failed to malloc CE_Acoustic_Table\n");
            return -1;
        }
    }

    current_str = read_buf;
    while (1) {
	    len=0;
        next_str = strchr(current_str, '\n');

        if (next_str == NULL)
        {
           break;
        }

        len = next_str - current_str;
        *next_str++ = '\0';
        if ( ParseAudioParaLine(current_str, len) < 0 ) {
            LOGI("ParseAudioParaLine failed :(\n");
            break;
        }
        current_str = next_str;
    }

//    munmap(read_buf, st.st_size);
    free(read_buf);
    close(csvfd);

    LOGI("Read:");
    LOGI("%d Audio_Path_Table entries", APT_max_index);
    LOGI("%d Audio_Path_Uplink_Table entries", APUT_max_index);
    LOGI("%d Phone_Acoustic_Table entries", PAT_max_index);
    LOGI("%d BT_Phone_Acoustic_Table entries", BTPAT_max_index);
    LOGI("%d HTC_VOC_CAL_CODEC_TABLE_Table entries", HVCCT_max_index);
    LOGI("%d CE_Acoustic_Table entries", CEAT_max_index);

    // initialise audio table with uplink off
    UpdateAudioAdieTable(0, 0, 0, 0, true);



    /* Table might need to be converted (on some devices, 1 setting is 8 params long,
     * whereas on some others, it's 0xB params long)
     * Note that this is the only one field size that varies between those devices.
     * All other field types are same size.
     */
    if ( device_capabilities.htc_voc_cal_fields_per_param < 0xB ) {
        int field;
        uint16_t* htc_voc_cal_tbl_conv_field;
        /* Convert table to required field size */
        htc_voc_cal_tbl_conv = (uint16_t*) malloc(HVCCT_max_index * device_capabilities.htc_voc_cal_fields_per_param * sizeof(uint16_t));
        if ( htc_voc_cal_tbl_conv == NULL ) {
            goto exit;
        }

        for (field=0; field<HVCCT_max_index; field++) {
            htc_voc_cal_tbl_conv_field = &htc_voc_cal_tbl_conv[field * device_capabilities.htc_voc_cal_fields_per_param];
            memcpy((void*) htc_voc_cal_tbl_conv_field,
                           HTC_VOC_CAL_CODEC_TABLE_Table[field].array,
                           device_capabilities.htc_voc_cal_fields_per_param * sizeof(uint16_t));
        }
        /* Fill the stucture with converted table that will be passed to kernel for update */
        htc_voc_cal_tbl.size = HVCCT_max_index * device_capabilities.htc_voc_cal_fields_per_param * sizeof(uint16_t);
        htc_voc_cal_tbl.pArray = htc_voc_cal_tbl_conv;
    } else {
        /* Fill the stucture with table that will be passed to kernel for update */
        htc_voc_cal_tbl.size = HVCCT_max_index * sizeof(struct d_table_s);
        htc_voc_cal_tbl.pArray = HTC_VOC_CAL_CODEC_TABLE_Table->array;
    }

    if (ioctl(acousticfd, ACOUSTIC_UPDATE_HTC_VOC_CAL_CODEC_TABLE,
                         &htc_voc_cal_tbl ) < 0) {
        LOGE("ACOUSTIC_UPDATE_HTC_VOC_CAL_CODEC_TABLE error.");
        return -EIO;
    }

    if ( htc_voc_cal_tbl_conv ) {
        free(htc_voc_cal_tbl_conv);
    }

exit:
/*
    if ( BT_Phone_Acoustic_Table[0] == 0 ) {
        memcpy(&BT_Phone_Acoustic_Table[0x40], &f_table[0x1680], 0x140);
    }
*/

    return 0;
}

int htc_acoustic_deinit(void)
{
    int rc = 0;

    /* Close the acoustic driver */
    close(acousticfd);

    /* Close TPA2016 */
    if ( device_capabilities.bUseTPA2016 ) {
        close(TPA2016fd);
    }

    /* Free the memory */
    if ( Audio_Path_Table != NULL )
        free(Audio_Path_Table);
    if ( Audio_Path_Uplink_Table != NULL )
        free(Audio_Path_Uplink_Table);
    if ( Phone_Acoustic_Table != NULL )
        free(Phone_Acoustic_Table);
    if ( BT_Phone_Acoustic_Table != NULL )
        free(BT_Phone_Acoustic_Table);
    if ( HTC_VOC_CAL_CODEC_TABLE_Table != NULL )
        free(HTC_VOC_CAL_CODEC_TABLE_Table);
    if ( mSndEndpoints != NULL )
        free(mSndEndpoints);

    return rc;
}

int msm72xx_set_acoustic_table(int device, int volume)
{
    struct fg_table_s* table = NULL;
    struct c_table_s*  ce_table = NULL;
    int out_path = device;
    int out_path_method = SND_METHOD_VOICE;

    LOGD("msm72xx_set_acoustic_table %d %d", device, volume);

/*
    if( device == SND_DEVICE_HANDSET ) {
        LOGD("Acoustic profile : EARCUPLE");
        out_path = EARCUPLE;
    } else if( device == SND_DEVICE_SPEAKER ) {
        LOGD("Acoustic profile : HANDSFREE");
        out_path = HANDSFREE;
    } else if( device == SND_DEVICE_HEADSET ) {
        LOGD("Acoustic profile : HEADSET");
        out_path = HEADSET;
    } else if( device == SND_DEVICE_NO_MIC_HEADSET ) {
        LOGD("Acoustic profile : HEADSET");
        out_path = HEADSET;
    } else if( device == SND_DEVICE_HEADSET_AND_SPEAKER ) {
        LOGD("Acoustic profile : HEADSET");
        out_path = HEADSET;
    } else if( (device == SND_DEVICE_BT) || (device == SND_DEVICE_BT_EC_OFF) ) {
        LOGD("Acoustic profile : BTHEADSET");
        out_path = BTHEADSET;
    } else if( device == SND_DEVICE_CARKIT ) {
        LOGD("Acoustic profile : CARKIT");
        out_path = CARKIT;
    } else if( device == SND_DEVICE_TTY_FULL ) {
        LOGD("Acoustic profile : TTY_FULL");
        out_path = TTY_FULL;
    } else if( device == SND_DEVICE_TTY_VCO ) {
        LOGD("Acoustic profile : TTY_VCO");
        out_path = TTY_VCO;
    } else if( device == SND_DEVICE_TTY_HCO ) {
        LOGD("Acoustic profile : TTY_HCO");
        out_path = TTY_HCO;

    } else if( device == SND_DEVICE_PLAYBACK_HEADSET ) {
        LOGV("Acoustic profile : PLAYBACK_HEADSET");
        out_path = PLAYBACK_HEADSET;
    } else if( device == SND_DEVICE_PLAYBACK_HANDSFREE ) {
        LOGV("Acoustic profile : PLAYBACK_HANDSFREE");
        out_path = PLAYBACK_HANDSFREE;

//    } else if( device == SND_DEVICE_REC_INC_MIC ) {
 //       LOGV("Acoustic profile : REC_INC_MIC");
  //      out_path = REC_INC_MIC;
    } else if( device == SND_DEVICE_IDLE ) {
        LOGV("Acoustic profile : EARCUPLE");
        out_path = EARCUPLE;
 //   } else if ( device >= BT_CUSTOM_DEVICES_ID_OFFSET ) {
 //       LOGV("Acoustic profile : CUSTOM_BTHEADSET");
 //       out_path = CUSTOM_BTHEADSET;

*/
/*
    switch ( out_path )
    {
//            table = &Phone_Acoustic_Table[(out_path*6) + volume];
        case HEADSET:
            table = &Phone_Acoustic_Table[0 + volume];
	break;
        case HANDSFREE:
            table = &Phone_Acoustic_Table[6 + volume];
	break;
        case EARCUPLE:
            table = &Phone_Acoustic_Table[12 + volume];
        break;

        case BTHEADSET:
            table = &Phone_Acoustic_Table[18];
        break;

        case CUSTOM_BTHEADSET:
            table = &BT_Phone_Acoustic_Table[device - BT_CUSTOM_DEVICES_ID_OFFSET];
            out_path = BTHEADSET;
        break;

        case CARKIT:
            table = &Phone_Acoustic_Table[19];
        break;

        case TTY_FULL:
            table = &Phone_Acoustic_Table[20];
            out_path_method = SND_METHOD_NONE;
        break;

        case TTY_VCO:
            table = &Phone_Acoustic_Table[21];
            out_path_method = SND_METHOD_NONE;
        break;

        case TTY_HCO:
            table = &Phone_Acoustic_Table[22];
            out_path_method = SND_METHOD_NONE;
        break;

        case REC_INC_MIC:
            table = &Phone_Acoustic_Table[23];
            out_path_method = SND_METHOD_AUDIO;
        break;

        case REC_EXT_MIC:
            table = &Phone_Acoustic_Table[24];
            out_path_method = SND_METHOD_AUDIO;
        break;

        case PLAYBACK_HEADSET:
            table = &Phone_Acoustic_Table[25];
            out_path_method = SND_METHOD_AUDIO;
        break;

        case PLAYBACK_HANDSFREE:
            table = &Phone_Acoustic_Table[26];
            out_path_method = SND_METHOD_AUDIO;
        break;

        default:
            LOGE("Unknown out_path");
        break;
    }
*/

	return 0;
}

int msm72xx_set_acoustic_done(void)
{
    LOGV("msm72xx_set_acoustic_done");

    if (ioctl(acousticfd, ACOUSTIC_ARM11_DONE, NULL) < 0) {
        LOGE("ACOUSTIC_ARM11_DONE error.");
        return -EIO;
    }
    return 0;
}

int set_acoustic_parameters(void)
{
    LOGV("set_acoustic_parameters called\n");

    if(ReadAudioParaFromFile() < 0)
    {
        LOGE("ReadAudioParaFromFile failed\n");
        return -1;
    }

    if(msm72xx_set_acoustic_done() < 0)
    {
        LOGE("msm72xx_set_acoustic_done failed\n");
        return -1;
    }

    htc_acoustic_deinit();

    return 0;
} 

}; // namespace android
